package ru.myitschool.work.data

import ru.myitschool.work.data.local.CredentialsLocalDataSource
import ru.myitschool.work.data.local.UserLocalDataSource
import ru.myitschool.work.domain.entities.UserEntity
import ru.myitschool.work.domain.login.LoginRepository
import ru.myitschool.work.data.network.UserNetworkDataSource
import ru.myitschool.work.domain.user.UserRepository

class UserRepositoryImpl(
    private val credentialsLocalDataSource: CredentialsLocalDataSource,
    private val userLocalDatSource: UserLocalDataSource,
    private val networkDataSource: UserNetworkDataSource
) : LoginRepository, UserRepository {

    override suspend fun login(login: String, password: String): Result<Unit> {

        return runCatching {
            networkDataSource.login(credentialsLocalDataSource.setAuthData(login, password))
                .onSuccess { dto ->
                    userLocalDatSource.cacheData(
                        UserEntity(
                            id = dto.id ?: error("Null user id"),
                            name = dto.name ?: error("Null user name"),
                            lastVisit = dto.lastVisit ?: error("Null user lastVisit"),
                            photoUrl = dto.photoUrl ?: error("Null user photoUrl"),
                            position = dto.position ?: error("Null user position")
                        )
                    )
                }
        }
    }

    override suspend fun logout() {
        credentialsLocalDataSource.clear()
        userLocalDatSource.clear()
    }

    override suspend fun isUserExist(login: String): Result<Boolean> {
        return networkDataSource.isUserExist(login)
    }

    override suspend fun getCurrentUser(): UserEntity {
        return userLocalDatSource.getUser()!!
    }
}